import sys

from prompt_toolkit import HTML, print_formatted_text, PromptSession
from prompt_toolkit.output import ColorDepth as Depth

from NWSh.settings import Settings
from NWSh.subsystem import Subsystem


class System(Subsystem):
    commands = {}

    def __init__(self, name: str = "(none)", preferences=None):
        super().__init__(name, preferences)
        self.prompt = f"{name}> "
        self.session = PromptSession()
        self.settings = Settings()
        self.style = None
        self.number = 0
        self.get_style()

        # language=HTML
        self.ctrl_c_msg = HTML(
            f"<system_name>{self.name}</system_name>: Press <b>Ctrl</b>+<b>D</b> (EOF) to exit."
        )

    def ask_command(self):
        # language=HTML
        self.prompt = HTML(
            f"<system_name>{self.name}</system_name> "
            f"<number>[{self.number}]</number>"
            f"<prompt>></prompt> "
        )
        self.number += 1
        try:
            result = self.session.prompt(self.prompt, style=self.style, color_depth=Depth.TRUE_COLOR)
        except KeyboardInterrupt:
            print_formatted_text(self.ctrl_c_msg, style=self.style)
            return
        except EOFError:
            sys.exit(0)

        self.execute_command(result)



import os
from pathlib import Path

from NWSh.constants import OSX, WINDOWS

# Path to the user preferences folder
if WINDOWS:
    USER_PREFS_DIR = Path(os.getenv("APPDATA")) / "NWSOFT" / "NWSh"
elif OSX:
    USER_PREFS_DIR = Path.home() / "Library" / "Preferences" / "NWSOFT" / "NWSh"
else:
    USER_PREFS_DIR = Path.home() / ".NWSOFT" / "NWSh"

# Path to the user preferences file
USER_PREFS_FILE = USER_PREFS_DIR / "settings.json"
DEFAULT_PREFS_FILE = Path(__file__).parent / "resources" / "settings.json"



from prompt_toolkit import HTML, print_formatted_text
from prompt_toolkit.styles import Style

from NWSh.settings import Settings


def print_error(subsystem: str, error: str):
    settings = Settings()
    using_style = settings.get("styles")[settings.get("using")]
    style = Style.from_dict(using_style)
    # language=HTML
    print_formatted_text(
        HTML(f"<subsystem_name>{subsystem}</subsystem_name>|<error>ERR : {error}</error>"), style=style
    )


def print_warning(subsystem: str, warning: str):
    settings = Settings()
    using_style = settings.get("styles")[settings.get("using")]
    style = Style.from_dict(using_style)
    # language=HTML
    print_formatted_text(
        HTML(f"<subsystem_name>{subsystem}</subsystem_name>|<warning>WARN: {warning}</warning>"), style=style
    )


def print_info(subsystem: str, info: str):
    settings = Settings()
    using_style = settings.get("styles")[settings.get("using")]
    style = Style.from_dict(using_style)
    # language=HTML
    print_formatted_text(HTML(f"<subsystem_name>{subsystem}</subsystem_name>|<info>INFO: {info}</info>"), style=style)


def print_result(subsystem: str, result: str):
    settings = Settings()
    using_style = settings.get("styles")[settings.get("using")]
    style = Style.from_dict(using_style)
    # language=HTML
    print_formatted_text(
        HTML(f"<subsystem_name>{subsystem}</subsystem_name>|<result>=> {result}</result>"), style=style
    )



import sys

from prompt_toolkit import HTML, print_formatted_text, PromptSession
from prompt_toolkit.output.color_depth import ColorDepth as Depth
from prompt_toolkit.styles import Style

from NWSh.printing import print_error
from NWSh.settings import Settings

DEFAULT_PREFERENCES = {
    "name"       : "(none)",
    "version"    : "0.1",
    "description":  # language=HTML
    "<i>No description</i>",
    "author"     : "Anonymous",
    "license"    : "MIT",
}


class Subsystem:
    commands = {
        "exit": lambda: sys.exit(0)
    }

    def __init__(self, name: str = "(none)", preferences=None):
        if preferences is None:
            preferences = {}
        self.preferences = DEFAULT_PREFERENCES | preferences
        self.name = name
        self.prompt = f"{name}> "
        self.session = PromptSession()
        self.settings = Settings()
        self.style = None
        self.number = 0
        self.get_style()

        # language=HTML
        self.ctrl_c_msg = HTML(
            f"<subsystem_name>{self.name}</subsystem_name>: Press <b>Ctrl</b>+<b>D</b> (EOF) to exit."
        )

        self.print_welcome()

    def print_welcome(self):
        # language=HTML
        print_formatted_text(
            HTML(
                f"""\
<title><b>{self.preferences["name"]}</b> version {self.preferences["version"]}
By {self.preferences["author"]}, under the {self.preferences["license"]} license
</title>\
<description>\
{self.preferences["description"]}
</description>\
            """,
            ),
            style=self.style,
        )

    def get_style(self):
        self.style = Style.from_dict(
            self.settings.get("styles")
            [self.settings.get("using")]
        )

    def command_not_found(self):
        print_error(self.name, "Command not found")

    def execute_command(self, name):
        try:
            command_function = self.commands[name]
        except KeyError:
            command_function = self.command_not_found
        command_function()

    def ask_command(self):
        self.prompt = HTML(
            # language=HTML
            f"<subsystem_name>({self.name})</subsystem_name> "
            f"<number>[{self.number}]</number>"
            f"<prompt>></prompt> "
        )
        self.number += 1
        try:
            result = self.session.prompt(self.prompt, style=self.style, color_depth=Depth.TRUE_COLOR)
        except KeyboardInterrupt:
            print_formatted_text(self.ctrl_c_msg, style=self.style)
            return
        except EOFError:
            sys.exit(0)

        self.execute_command(result)





from typing import Dict, Union

from prompt_toolkit import HTML

from NWSh.printing import print_error
from NWSh.subsystem import Subsystem


class Arguments(Subsystem):
    def __init__(self, name: str = "(none)", preferences=None):
        super().__init__(name, preferences)
        self.arguments: Dict[str, Union[str, int, float]] = {}

    def print_welcome(self):
        return  # Do not print welcome message, as this is not a "normal" subsystem

    def ask_argument(self, argument_name: str, argument_description: str, argument_type: str = "str"):
        # language=HTML
        self.prompt = HTML(
            f"<subsystem_name>{self.name}</subsystem_name>"
            f"<number>({argument_name})</number>"
            f"<prompt>></prompt> "
        )
        self.arguments[argument_name] = self.session.prompt(self.prompt, rprompt=argument_description, style=self.style)

        if argument_type == "int":
            try:
                self.arguments[argument_name] = int(self.arguments[argument_name])
            except ValueError:
                print_error(self.name, f"Argument {argument_name} must be an integer")
                self.ask_argument(argument_name, argument_description, argument_type)
        elif argument_type == "float":
            try:
                self.arguments[argument_name] = float(self.arguments[argument_name])
            except ValueError:
                print_error(self.name, f"Argument {argument_name} must be a float")
                self.ask_argument(argument_name, argument_description, argument_type)

    def get_argument(self, argument_name: str):
        try:
            return self.arguments[argument_name]
        except KeyError:
            print_error(self.name, f"Argument {argument_name} not found")
            return None



from typing import Callable

from NWSh.printing import print_warning
from NWSh.subsystem import Subsystem
from NWSh.system import System


def register_command_system(system: System, command: str, func: Callable):
    if command in system.commands:
        print_warning(system.name, f"Command {command} already registered")
    system.commands[command] = func


def register_command_subsystem(subsystem: Subsystem, command: str, func: Callable):
    if command in subsystem.commands:
        print_warning(subsystem.name, f"Command {command} already registered")
    subsystem.commands[command] = func



import json5rw as json

from NWSh.paths import DEFAULT_PREFS_FILE, USER_PREFS_FILE


class Settings:
    def __init__(self):
        self.settings = {}
        self.load()

    def load(self):
        try:
            with DEFAULT_PREFS_FILE.open() as f:
                self.settings = json.load(f)
            with USER_PREFS_FILE.open() as f:
                self.settings |= json.load(f)
        except FileNotFoundError:
            self.save()

    def save(self):
        USER_PREFS_FILE.parent.mkdir(parents=True, exist_ok=True)
        with USER_PREFS_FILE.open("w") as f:
            json.dump(self.settings, f, indent=4, sort_keys=True)

    def get(self, key, default=None):
        return self.settings.get(key, default)

    def set(self, key, value):
        self.settings[key] = value
        self.save()



from setuptools import setup
# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='NWSh',
    version='0.1.3',
    packages=['NWSh'],
    url='https://www.nwsoft.tk',
    license='GPL',
    author='ZCG-Coder',
    author_email='andy@nwsoft.tk',
    description='A little library for the NWSh development',
    install_requires=[
        'prompt_toolkit',
        'json5rw',
    ],
    data_files=[('NWSh/resources', ['NWSh/resources/settings.json'])],
    include_dirs=['NWSh/resources'],
    include_package_data=True,
    long_description=long_description,
    long_description_content_type='text/markdown'
)
